local brute = require "brute"
local creds = require "creds"
local nmap = require "nmap"
local shortport = require "shortport"
local stdnse = require "stdnse"
local string = require "string"
local ads = require "ads"
local nsedebug = require "nsedebug"

description = [[
Attempts to find the username / password combination to establish an ADS/AMS route.

The script sends an ADS packet with credentials to the target system and analyzes the 
response. If an error code is returned, a new packet with different credentials is sent,
until the error code 0x00 is returned.
]]

---
-- @usage
-- nmap -sn --script ads-brute <target> --script-args 'port=48899,threads=16,username=admin'
--
-- @output
-- Host script results:
-- | ads-brute:
-- |
-- |     Accounts
-- |       admin:letmein - Valid credentials
-- |     Statistics
-- |_      Performed 65038 guesses in 75 seconds, average tps: 866

author = "Joel Spang"
license = "Same as Nmap--See http://nmap.org/book/man-legal.html"
categories = {"intrusive", "brute"}

hostrule = function() return true end

Driver = {
  new = function(self, host, port, options)
    local o = {}
    setmetatable(o, self)
    self.__index = self
    o.host = host
    o.port = port
    o.options = options
    o.firstonly = false
    o.username = self.username
    return o
  end,
  connect = function( self )
    if not self.socket then
      self.socket = nmap.new_socket()
    end

    return self.socket:connect( self.host, self.port )
  end,
  disconnect = function( self )
    if self.socket then
      return self.socket:close()
    end
    return true
  end,
  check = function( self )
    return true
  end,
  login = function( self, username, password )
    if self.username and self.username ~= username then return false, brute.Error:new( "skipping username" ) end

    local status = ads.send_route_establish(self.host, self.port, self.username or username, password, nil)
    if ( status ) then
      return true, brute.Account:new(self.username or username, password, creds.State.VALID)
    else
      return false, brute.Error:new( "login failed" )
    end
  end,  
}



local credentials_table = {}

action = function(host, port) 
  local args = nmap.registry.args
  local port = {
    number = (args.port ~= nil and tonumber(args.port)) or 48899,
    protocol = "udp"
  }

  Driver.username = (args.username ~= nil and tostring(args.username) ~= "0" and tostring(args.username)) or false

  local engine = brute.Engine:new(Driver, host, port)
  engine.port = port
  engine.options.script_name = SCRIPT_NAME

  local threadCount = (args.threads ~= nil and tonumber(args.threads)) or 8
  nmap.registry.args["brute.threads"] = threadCount
  engine:setMaxThreads(threadCount)

  status, accounts = engine:start()

  if( not(status) ) then
    return accounts
  end

  return stdnse.format_output( true, accounts )
end